"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AdmZip = require("adm-zip");
const nodelogging_1 = require("@hibas123/nodelogging");
const Logging = new nodelogging_1.LoggingBase("@hibas123/epubparser");
const xml2js = require("xml2js");
function parseXML(data) {
    return new Promise((yes, no) => xml2js.parseString(data, (err, res) => err ? no(err) : yes(res)));
}
class EpubParser {
    constructor(file) {
        this.file = file;
    }
    async parse() {
        const z = new AdmZip(this.file);
        const getData = (name) => {
            let entry = z.getEntry(name);
            if (!entry) {
                throw new Error("Cannot find entry:" + name);
            }
            return new Promise((yes, no) => entry.getDataAsync(yes));
        };
        const getDataS = (name) => getData(name).then(r => r.toString("utf8"));
        const mimetype = await getDataS("mimetype");
        Logging.debug("MimeType: ", mimetype);
        if (mimetype !== "application/epub+zip")
            throw new Error("Invalid mimetype!");
        const { container } = await parseXML(await getDataS("META-INF/container.xml"));
        Logging.debug("Getting content.opf");
        const contentFile = container.rootfiles[0].rootfile.find(e => e.$["media-type"] === "application/oebps-package+xml").$["full-path"];
        const content = await getDataS(contentFile);
        const parsedContent = await parseXML(content);
        const metadata = parsedContent.package.metadata[0];
        const getMetaValue = (field) => {
            let m = metadata[field];
            if (Array.isArray(m)) {
                m = m[0];
            }
            ;
            if (typeof m === "object") {
                m = m._;
            }
            return m;
        };
        const title = getMetaValue("dc:title");
        const creator = getMetaValue("dc:creator");
        const summary = getMetaValue("dc:description");
        const id = getMetaValue("dc:identifier");
        const manifest = parsedContent.package.manifest[0];
        const items = manifest.item;
        const itemMap = new Map();
        let ncxFile;
        items.map(e => e.$).forEach(item => {
            const type = item["media-type"];
            const id = item["id"];
            const href = item["href"];
            itemMap.set(id, { type, href });
            if (type === "application/x-dtbncx+xml") {
                ncxFile = href;
            }
        });
        const { ncx } = await parseXML(await getDataS(ncxFile));
        const chapters = await Promise.all(ncx.navMap[0].navPoint
            .filter(point => point.$.class === "chapter")
            .map(point => {
            const title = point.navLabel[0].text[0];
            const src = point.content[0].$.src.split("#")[0];
            return { title, src };
        }).map(async ({ title, src }) => {
            const content = await getDataS(src);
            return {
                title,
                content
            };
        }));
        return {
            id,
            title,
            creator,
            summary,
            getFile: (href) => {
                return getData(href);
            },
            getFileByID: (id) => {
                Logging.debug("Get File with id:", id);
                let item = itemMap.get(id);
                Logging.debug(item);
                if (item) {
                    return {
                        getData: () => getData(item.href),
                        type: item.type
                    };
                }
                return undefined;
            },
            chapters
        };
    }
}
exports.default = EpubParser;
//# sourceMappingURL=index.js.map